@@ -160,7 +160,7 @@ fn level_dat() {
     let validate_nbt = io::read_nbt(&mut Cursor::new(LEVEL_DAT), Flavor::GzCompressed)
         .unwrap()
         .0;
-    assert_eq!(test_nbt, validate_nbt)
+    assert_compound_eq!(test_nbt, validate_nbt)
 }
 
 #[test]
@@ -178,6 +178,7 @@ fn basic_datatypes_serde() {
         string: String,
     }
 
+    #[allow(clippy::excessive_precision, clippy::approx_constant)]
     let test_struct = Foo {
         boolean: true,
         unsigned_byte: 174,
@@ -199,6 +200,7 @@ fn basic_datatypes_serde() {
     .unwrap()
     .0;
 
+    #[allow(clippy::excessive_precision, clippy::approx_constant)]
     let validation_nbt = compound! {
         "boolean": 1i8,
         "unsigned_byte": -82i8,
@@ -211,7 +213,7 @@ fn basic_datatypes_serde() {
         "string": "Mario"
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: Foo = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -266,7 +268,7 @@ fn complex_structs_serde() {
         "b": [12i8, 12i8, 12i8]
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: Foo = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -329,7 +331,7 @@ fn enum_serde() {
         "unit": 4i32
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: Foo = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -412,7 +414,7 @@ fn array_serde() {
         "prim_long_array": [L; i64::MIN, i64::MAX, 0]
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: Foo = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -425,7 +427,7 @@ fn array_serde() {
     impl<'a> Serialize for Bytes<'a> {
         fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
         where S: serde::Serializer {
-            serializer.serialize_bytes(&self.0)
+            serializer.serialize_bytes(self.0)
         }
     }
 
@@ -573,7 +575,7 @@ fn array_serde() {
         Ok(NbtTag::IntArray(_))
     ));
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: SerAsSlice = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -631,10 +633,10 @@ fn vec_serde() {
         strings: vec!["test".to_owned(), "test test test".to_owned()],
         enumeration: vec![Enumeration::A, Enumeration::B, Enumeration::E],
         mixed_enumeration: vec![Enumeration::D(12), Enumeration::F { a: 14 }],
-        nested_arr: vec![
-            vec![vec![1, 20, 9].into()].into(),
-            vec![vec![3, 5, 10].into(), vec![99, 10, 32].into()].into(),
-        ],
+        nested_arr: vec![vec![vec![1, 20, 9].into()], vec![
+            vec![3, 5, 10].into(),
+            vec![99, 10, 32].into(),
+        ]],
         enum_of_vec: Enumeration::G(vec![vec![Bar { a: 13 }, Bar { a: 9 }], vec![Bar { a: 14 }]]),
         empty_byte_array: Vec::new().into(),
         empty_int_array: Vec::new().into(),
@@ -716,7 +718,7 @@ fn vec_serde() {
         "seq_empty_tag_list": [[], []],
     };
 
-    assert_eq!(nbt_struct, validation_nbt);
+    assert_compound_eq!(nbt_struct, validation_nbt);
 
     let deserialized_struct: Foo = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -782,7 +784,7 @@ fn option_serde() {
         "g": [21i8, 98i8]
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: Foo = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
@@ -828,7 +830,7 @@ fn borrowed_serde() {
         }
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: BorrowedData<'_> =
         deserialize_from_buffer(&serialized_struct).unwrap().0;
@@ -866,6 +868,7 @@ fn inlined_nbt() {
         .unwrap()
         .clone();
 
+    #[allow(clippy::excessive_precision, clippy::approx_constant)]
     let inlined = Inlined {
         byte: NbtTag::Byte(31),
         short: NbtTag::Short(-214),
@@ -892,6 +895,7 @@ fn inlined_nbt() {
     .unwrap()
     .0;
 
+    #[allow(clippy::excessive_precision, clippy::approx_constant)]
     let validation_nbt = compound! {
         "byte": 31i8,
         "short": -214i16,
@@ -910,7 +914,7 @@ fn inlined_nbt() {
         "normal_field": 42i32
     };
 
-    assert_eq!(struct_nbt, validation_nbt);
+    assert_compound_eq!(struct_nbt, validation_nbt);
 
     let deserialized_struct: Inlined = deserialize(&serialized_struct, Flavor::Uncompressed)
         .unwrap()
